/*
 * SPDX-FileCopyrightText: 2017 Elvis Angelaccio <elvis.angelaccio@kde.org>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 *
 */

#include "abstractaccountmanager.h"

AbstractAccountManager::~AbstractAccountManager()
{
}
